@@ -41,16 +41,15 @@ async fn main() -> Result<()> {
     let args: Vec<String> = env::args().collect();
 
     // create an endpoint for us to listen on and send from.
-    let (node, _incoming_conns, mut incoming_messages, _disconnections, _contact) =
-        Endpoint::<XId>::new(
-            SocketAddr::from((Ipv4Addr::LOCALHOST, 0)),
-            &[],
-            Config {
-                idle_timeout: Duration::from_secs(60 * 60).into(), // 1 hour idle timeout.
-                ..Default::default()
-            },
-        )
-        .await?;
+    let (node, _incoming_conns, mut incoming_messages, _contact) = Endpoint::<XId>::new(
+        SocketAddr::from((Ipv4Addr::LOCALHOST, 0)),
+        &[],
+        Config {
+            idle_timeout: Duration::from_secs(60 * 60).into(), // 1 hour idle timeout.
+            ..Default::default()
+        },
+    )
+    .await?;
 
     // if we received args then we parse them as SocketAddr and send a "marco" msg to each peer.
     if args.len() > 1 {
