@@ -60,7 +60,7 @@ mod wire_msg;
 
 pub use config::{Config, ConfigError, RetryConfig};
 pub use connection::{RecvStream, SendStream};
-pub use connection_handle::{ConnectionHandle as Connection, DisconnectionEvents};
+pub use connection_handle::ConnectionHandle as Connection;
 pub use connection_pool::ConnId;
 pub use endpoint::{Endpoint, IncomingConnections, IncomingMessages};
 #[cfg(feature = "igd")]
